"""
April 19th, 2010
Institute for Genomics and Bioinformatics, University of Caifornia Irvine

This file contains Python code that implements the probability density function (pdf) of the ratio of two
correlated normal variables used in the manuscript: "When is Chemical Similarity Significant? The Statistical 
Distribution of Chemical Similarity Scores and Its Extreme Values" Submitted to JCIM 2010.
The pdf is given by Equations 2-6 in the manuscript.
The function gaussRatioProb gives the probability density at a point z (z=X/Y)
given 5 parameters: mu_X, mu_Y, sigma_X, sigma_Y, and rho, and a normalizing factor (norm).
mu_X and sigma_X are the mean and standard deviation of the numerator random variable (X).
mu_Y and sigma_Y are the mean and standard deviation of the denominator random variable (Y).
rho is the correlation between the two random variables X and Y.
The integration of this function over an interval (using, for example, built-in functions in the scipy package)
gives the probabilty of z belonging to that interval. 
"""

def gaussRatioProb(z, mu_X, mu_Y, sigma_X, sigma_Y, rho, norm=1):
    """Probability for Z=X/Y where X and Y are two correlated normal random variables.
        Inputs:
            z = dependent variable
            mu_X, mu_Y = means of X and Y
            sigma_X, sigma_Y = std of X and Y
            rho = correlation between X and Y
            norm = optional factor to divide the result by; useful for normalizing
                    the probability if the density goes below zero.
    """
    # The probability density is defined in terms of 3 main quantities, Rz, supR2 and supR2_R2,
    # which are themselves computed from smaller building blocks

    #calculating Rz (Equation 4 in the manuscript)
    Rz_1 = (sigma_Y**2 * mu_X - rho*sigma_X*sigma_Y*mu_Y) * z - rho*sigma_X*sigma_Y*mu_X + sigma_X**2 * mu_Y
    Rz_2 = sigma_X*sigma_Y * sqrt(1-rho**2) * sqrt(sigma_Y**2 * z**2 - 2*rho*sigma_X*sigma_Y*z + sigma_X**2)
    Rz = Rz_1 / Rz_2
    
    #calculating supR2 (Equation 5 in the manuscript)
    supR2_1 = (sigma_Y**2 * mu_X**2 - 2*rho*sigma_X*sigma_Y*mu_X*mu_Y + sigma_X**2*mu_Y**2)
    supR2_2 = sigma_X**2 * sigma_Y**2 * (1-rho**2)
    supR2 = supR2_1 / supR2_2
    
    #calculating supR2 - R2 (Equation 6 in the manuscript)
    supR2_R2 = (mu_X - mu_Y*z)**2 / (sigma_Y**2 * z**2 - 2*rho*sigma_X*sigma_Y*z + sigma_X**2)
    
    #calculating P(z) (Equation 3 in the manuscript)
    Pz_1 = sigma_X*sigma_Y * sqrt(1-rho**2) / (pi * (sigma_Y**2 * z**2 - 2*rho*sigma_X*sigma_Y*z + sigma_X**2))
    Pz_2 = exp(-0.5 * supR2) + sqrt(2*pi) * Rz * Phi(Rz) * exp(-0.5*supR2_R2)
    Pz = Pz_1 * Pz_2
    
    return Pz/norm

#function called within gaussRatioProb
def Phi(x):
    """Calculation of Error function in Gauss form"""
    return 0.5 * erf(x/sqrt(2))

